@@ -40,8 +40,8 @@
 #include <KSharedConfig>
 #include <KPluginInfo>
 #include <KComboBox>
-#include <KPassivePopup>
 #include <KConfigDialog>
+#include <KWindowSystem>
 
 #include <QTextStream>
 #include <QDomElement>
@@ -55,7 +55,12 @@
 #endif
 
 #ifdef HAVE_KWORKSPACE
+    #include <QDBusConnection>
+    #include <QDBusInterface>
+    #include <QDBusPendingCall>
     #include <kworkspace/kworkspace.h>
+    #include <solid/control/powermanager.h>
+    #include <solid/powermanagement.h>
 #endif
 
 /**
@@ -255,10 +260,6 @@ TransferHandler * KGet::addTransfer(const QDomElement& e, const QString& groupNa
               << " dest= " << destUrl.url() 
               << " group= "<< groupName << endl;
 
-    if ( srcUrl.isEmpty() || !isValidSource(srcUrl) 
-         || !isValidDestDirectory(destUrl.directory()) )
-        return 0;
-
     return createTransfer(srcUrl, destUrl, groupName, false, &e);
 }
 
@@ -330,7 +331,10 @@ bool KGet::delTransfer(TransferHandler * transfer)
 {
     Transfer * t = transfer->m_transfer;
     t->stop();
-    TransferHistoryStore::getStore()->saveItem(TransferHistoryItem(*t));
+    if (!m_store) {
+        m_store = TransferHistoryStore::getStore();
+    }
+    m_store->saveItem(TransferHistoryItem(*t));
 
     // TransferHandler deinitializations
     transfer->destroy();
@@ -523,10 +527,8 @@ void KGet::save( QString filename, bool plain ) // krazy:exclude=passbyvalue
     if ( !file.open( QIODevice::WriteOnly ) )
     {
         //kWarning(5001)<<"Unable to open output file when saving";
-        KMessageBox::error(0,
-                        i18n("Unable to save to: %1", filename),
-                        i18n("Error"));
-
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Unable to save to: %1", filename));
         return;
     }
 
@@ -728,6 +730,7 @@ QList<TransferFactory *> KGet::m_transferFactories;
 TransferGroupScheduler * KGet::m_scheduler = new TransferGroupScheduler();
 MainWindow * KGet::m_mainWindow = 0;
 KUiServerJobs * KGet::m_jobManager = 0;
+TransferHistoryStore * KGet::m_store = 0;
 
 // ------ PRIVATE FUNCTIONS ------
 KGet::KGet()
@@ -759,6 +762,7 @@ KGet::~KGet()
     delete m_transferTreeModel;
     delete m_jobManager;  //This one must always be before the scheduler otherwise the job manager can't remove the notifications when deleting.
     delete m_scheduler;
+    delete m_store;
 }
 
 TransferHandler * KGet::createTransfer(const KUrl &src, const KUrl &dest, const QString& groupName, 
@@ -785,6 +789,10 @@ TransferHandler * KGet::createTransfer(const KUrl &src, const KUrl &dest, const
             newTransfer->create();
             newTransfer->load(e);
             m_transferTreeModel->addTransfer(newTransfer, group);
+            
+            KGet::showNotification(m_mainWindow, "added",
+                                   i18n("<p>The following transfer has been added to the download list:</p><p style=\"font-size: small;\">\%1</p>", newTransfer->source().pathOrUrl()),
+                                   "kget", i18n("Download added"));
 
             if(start)
                 newTransfer->handler()->start();
@@ -793,9 +801,8 @@ TransferHandler * KGet::createTransfer(const KUrl &src, const KUrl &dest, const
         }
     }
     
-    KMessageBox::error(m_mainWindow, 
-                       i18n("Unable to continue: KGet cannot download using this protocol."),
-                       i18n("Protocol unsupported"));
+    KGet::showNotification(m_mainWindow, "error",
+                           i18n("Unable to continue: KGet cannot download using this protocol."), "dialog-error", i18n("Protocol unsupported"));
 
     kDebug(5001) << "Warning! No plugin found to handle the given url";
     return 0;
@@ -817,22 +824,12 @@ TransferDataSource * KGet::createTransferDataSource(const KUrl &src, const QDomE
 
 QString KGet::generalDestDir(bool preferXDGDownloadDir)
 {
-    QString dir;
+    QString dir = Settings::lastDirectory();
 
     if (preferXDGDownloadDir) {
         dir = KGlobalSettings::downloadPath();
     }
 
-    if (dir.isEmpty()) {
-        dir = Settings::lastDirectory();
-        if (dir.isEmpty()) {
-            dir = KGlobalSettings::desktopPath();
-            if (dir.isEmpty()) {
-                dir = QDir::homePath();
-            }
-        }
-    }
-
     return dir;
 }
 
@@ -878,23 +875,17 @@ KUrl KGet::destFileInputDialog(QString destDir, const QString& suggestedFileName
     if (destDir.isEmpty())
         destDir = generalDestDir();
 
-    // open the filedialog for confirmation
-    QPointer<KFileDialog> dlg = new KFileDialog(destDir, QString(), 0, 0);
-    dlg->setCaption(i18n("Save As"));
-    dlg->setOperationMode(KFileDialog::Saving);
-    dlg->setMode(KFile::File | KFile::LocalOnly);
-
     // Use the destination name if not empty...
-    if (!suggestedFileName.isEmpty())
-        dlg->setSelection(suggestedFileName);
+    KUrl startLocation(destDir);
+    if (!suggestedFileName.isEmpty()) {
+        startLocation.addPath(suggestedFileName);
+    }
 
-    KUrl destUrl;
-    if (dlg->exec() != QDialog::Rejected){
-        destUrl = dlg->selectedUrl();
+    KUrl destUrl = KFileDialog::getSaveUrl(startLocation, QString(), m_mainWindow, i18n("Save As"));
+    if (!destUrl.isEmpty()) {
         Settings::setLastDirectory(destUrl.directory(KUrl::ObeyTrailingSlash));
     }
 
-    delete dlg;
     return destUrl;
 }
 
@@ -902,18 +893,15 @@ bool KGet::isValidSource(const KUrl &source)
 {
     // Check if the URL is well formed
     if (!source.isValid()) {
-        KMessageBox::error(0,
-                           i18n("Malformed URL:\n%1", source.prettyUrl()),
-                           i18n("Error"));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Malformed URL:\n%1", source.prettyUrl()));
 
         return false;
     }
     // Check if the URL contains the protocol
     if (source.protocol().isEmpty()){
-
-        KMessageBox::error(0, 
-                           i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()),
-                           i18n("Error"));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Malformed URL, protocol missing:\n%1", source.prettyUrl()));
 
         return false;
     }
@@ -1113,8 +1101,9 @@ KGetPlugin * KGet::createPluginFromService( const KService::Ptr &service )
 
     if (!factory)
     {
-        KMessageBox::error(0, i18n("<html><p>Plugin loader could not load the plugin:<br/><i>%1</i></p></html>",
-                                   service->library()));
+        KGet::showNotification(m_mainWindow, "error",
+                               i18n("Plugin loader could not load the plugin: %1.", service->library()),
+                               "dialog-info");
         kError(5001) << "KPluginFactory could not load the plugin:" << service->library();
         return 0;
     }
@@ -1130,7 +1119,7 @@ bool KGet::safeDeleteFile( const KUrl& url )
         QFileInfo info( url.toLocalFile() );
         if ( info.isDir() )
         {
-            KGet::showNotification(m_mainWindow, KNotification::Notification,
+            KGet::showNotification(m_mainWindow, "notification",
                                    i18n("Not deleting\n%1\nas it is a directory.", url.prettyUrl()),
                                    "dialog-info");
             return false;
@@ -1140,21 +1129,22 @@ bool KGet::safeDeleteFile( const KUrl& url )
     }
 
     else
-        KGet::showNotification(m_mainWindow, KNotification::Notification,
+        KGet::showNotification(m_mainWindow, "notification",
                                i18n("Not deleting\n%1\nas it is not a local file.", url.prettyUrl()),
                                "dialog-info");
     return false;
 }
 
-void KGet::showNotification(QWidget *parent, KNotification::StandardEvent eventId, 
-                            const QString &text, const QString &icon)
+KNotification *KGet::showNotification(QWidget *parent, const QString &eventType,
+                            const QString &text, const QString &icon, const QString &title, const KNotification::NotificationFlags &flags)
 {
-    KNotification::event(eventId, text, KIcon(icon).pixmap(KIconLoader::Small), parent);
+    return KNotification::event(eventType, title, text, KIcon(icon).pixmap(KIconLoader::SizeMedium), parent, flags);
 }
 
 GenericObserver::GenericObserver(QObject *parent)
   : QObject(parent),
-    m_save(0)
+    m_save(0),
+    m_finishAction(0)
 {
     connect(KGet::model(), SIGNAL(groupRemovedEvent(TransferGroupHandler*)), SLOT(groupRemovedEvent(TransferGroupHandler*)));
     connect(KGet::model(), SIGNAL(transferAddedEvent(TransferHandler*, TransferGroupHandler*)), 
@@ -1241,6 +1231,18 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
     {
         TransferHandler::ChangesFlags transferFlags = *it;
         TransferHandler *transfer = it.key();
+        
+        if (transferFlags & Transfer::Tc_Status) {
+            if ((transfer->status() == Job::Finished)   && (transfer->startStatus() != Job::Finished)) {
+                KGet::showNotification(KGet::m_mainWindow, "finished",
+                                       i18n("<p>The following file has finished downloading:</p><p style=\"font-size: small;\">\%1</p>", transfer->dest().fileName()),
+                                       "kget", i18n("Download completed"));
+            } else if (transfer->status() == Job::Running) {
+                KGet::showNotification(KGet::m_mainWindow, "started",
+                                       i18n("<p>The following transfer has been started:</p><p style=\"font-size: small;\">\%1</p>", transfer->source().pathOrUrl()),
+                                       "kget", i18n("Download started"));
+            }
+        }
 
         if (transferFlags & Transfer::Tc_Status)
             checkSysTray = true;
@@ -1258,27 +1260,61 @@ void GenericObserver::transfersChangedEvent(QMap<TransferHandler*, Transfer::Cha
             transfer->group()->setGroupChange(TransferGroup::Gc_UploadSpeed, true);
         }
 
-        if (transfer->status() == Job::Finished) {
+        if ((transfer->status() == Job::Finished) || (transfer->status() == Job::FinishedKeepAlive)) {
             requestSave();
         } else {
             allFinished = false;
         }
     }
+    allFinished = allFinished && allTransfersFinished();
 
     if (checkSysTray)
         KGet::checkSystemTray();
 
-    if (Settings::afterFinishActionEnabled())
+    if (Settings::afterFinishActionEnabled() && allFinished)
     {
-        if (allFinished)
-            kDebug() << "All finished";
-        if (allFinished && Settings::afterFinishAction() == KGet::Quit)
-            checkAndFinish();
+        kDebug(5001) << "All finished";
+        KNotification *notification = 0;
 
+        if (!m_finishAction) {
+            m_finishAction = new QTimer(this);
+            m_finishAction->setSingleShot(true);
+            m_finishAction->setInterval(10000);
+            connect(m_finishAction, SIGNAL(timeout()), this, SLOT(slotAfterFinishAction()));
+        }
+
+        switch (Settings::afterFinishAction()) {
+            case KGet::Quit:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("KGet is now closing, as all downloads have completed."), "kget", "KGet", KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
 #ifdef HAVE_KWORKSPACE
-        if (allFinished && Settings::afterFinishAction() == KGet::Shutdown)
-            checkAndShutdown();
+            case KGet::Shutdown:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now turn off, as all downloads have completed."), "system-shutdown", i18nc("Shutting down computer", "Shutdown"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
+            case KGet::Hibernate:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now suspend to disk, as all downloads have completed."), "system-suspend-hibernate", i18nc("Hibernating computer", "Hibernating"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
+            case KGet::Suspend:
+                notification = KGet::showNotification(KGet::m_mainWindow, "notification", i18n("The computer will now suspend to RAM, as all downloads have completed."), "system-suspend", i18nc("Suspending computer", "Suspending"), KNotification::Persistent | KNotification::CloseWhenWidgetActivated);
+                break;
 #endif
+            default:
+                break;
+        }
+
+        if (notification) {
+            notification->setActions(QStringList() << i18nc("abort the proposed action", "Abort"));
+            connect(notification, SIGNAL(action1Activated()), this, SLOT(slotAbortAfterFinishAction()));
+            connect(m_finishAction, SIGNAL(timeout()), notification, SLOT(close()));
+
+            if (!m_finishAction->isActive()) {
+                m_finishAction->start();
+            }
+        }
+    } else if (allFinished) {
+        KGet::showNotification(KGet::m_mainWindow, "finishedall",
+                               i18n("<p>All transfers have been finished.</p>"),
+                               "kget", i18n("Downloads completed"));
     }
 }
 
@@ -1307,11 +1343,11 @@ bool GenericObserver::allTransfersFinished()
 
     foreach(TransferGroup *transferGroup, KGet::model()->transferGroups()) {
         foreach(TransferHandler *transfer, transferGroup->handler()->transfers()) {
-            if(transfer->status() != Job::Finished) {
+            if ((transfer->status() != Job::Finished) && (transfer->status() != Job::FinishedKeepAlive)) {
                 quitFlag = false;
             }
-            if (transfer->status() == Job::Finished &&
-               transfer->startStatus() != Job::Finished)
+            if (((transfer->status() == Job::Finished) && (transfer->startStatus() != Job::Finished)) ||
+                ((transfer->status() == Job::FinishedKeepAlive) && (transfer->startStatus() != Job::FinishedKeepAlive)))
             {
                 allWereFinished = false;
             }
@@ -1331,51 +1367,45 @@ bool GenericObserver::allTransfersFinished()
     return quitFlag;
 }
 
-KPassivePopup* GenericObserver::popupMessage(const QString &title, const QString &message)
+void GenericObserver::slotAfterFinishAction()
 {
-    KPassivePopup *popup;
-    // we have to call diferent message from kpassivePopup
-    // one with parent as QWidget for the mainWindow
-    // and another with parent as QSystemTrayIcon if the parent is a systemTray
-    // so passing the QSystemTrayIcon as QWidget don't work
-    if(Settings::enableSystemTray()) 
-    {
-        popup = KPassivePopup::message(5000, title, message, KGet::m_mainWindow);
-    }
-    else 
-    {
-        popup = KPassivePopup::message(5000, title, message, KGet::m_mainWindow);
-    }
+    kDebug(5001);
 
-    return popup;
-}
-
-void GenericObserver::checkAndFinish()
-{
-    kDebug();
-    // check if there is some unfinished transfer in scheduler queues
-    if(allTransfersFinished()) {
-        KPassivePopup *message = popupMessage(i18n("Quit KGet"),
-                                            i18n("KGet is now closing, as all downloads have completed."));
-        QObject::connect(message, SIGNAL(destroyed()), KGet::m_mainWindow, SLOT(slotQuit()));
+    switch (Settings::afterFinishAction()) {
+        case KGet::Quit:
+            kDebug(5001) << "Quit Kget.";
+            QTimer::singleShot(0, KGet::m_mainWindow, SLOT(slotQuit()));
+            break;
+    #ifdef HAVE_KWORKSPACE
+        case KGet::Shutdown:
+            QTimer::singleShot(0, KGet::m_mainWindow, SLOT(slotQuit()));
+            KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmNo,
+                        KWorkSpace::ShutdownTypeHalt,
+                        KWorkSpace::ShutdownModeForceNow);
+            break;
+        case KGet::Hibernate: {
+            QDBusConnection dbus(QDBusConnection::sessionBus());
+            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+            iface.asyncCall("suspend", Solid::Control::PowerManager::ToDisk);
+            break;
+        }
+        case KGet::Suspend: {
+            QDBusConnection dbus(QDBusConnection::sessionBus());
+            QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+            iface.asyncCall("suspend", Solid::Control::PowerManager::ToRam);
+            break;
+        }
+    #endif
+        default:
+            break;
     }
 }
 
-#ifdef HAVE_KWORKSPACE
-void GenericObserver::checkAndShutdown()
+void GenericObserver::slotAbortAfterFinishAction()
 {
-    if(allTransfersFinished()) {
-        KPassivePopup *message = popupMessage(i18n("Quit KGet"),
-                                            i18n("The computer will now turn off, as all downloads have completed."));
-        QObject::connect(message, SIGNAL(destroyed()), SLOT(slotShutdown()));
-        QObject::connect(message, SIGNAL(destroyed()),  KGet::m_mainWindow, SLOT(slotQuit()));
-    }
+    kDebug(5001);
+
+    m_finishAction->stop();
 }
 
-void GenericObserver::slotShutdown()
-{
-    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmNo,
-                                KWorkSpace::ShutdownTypeHalt,
-                                KWorkSpace::ShutdownModeForceNow);
-}
-#endif
+#include "kget.moc"
