@@ -13,7 +13,7 @@
 #define TRANSFER_H
 
 #include "job.h"
-#include "kget_export.h"
+#include "../kget_export.h"
 
 #include <QPixmap>
 #include <QTime>
@@ -23,18 +23,21 @@
 
 class QDomElement;
 
+class Signature;
 class TransferHandler;
 class TransferFactory;
 class TransferGroup;
 class Scheduler;
 class TransferTreeModel;
 class NepomukHandler;
+class FileModel;
+class Verifier;
 
 class KGET_EXPORT Transfer : public Job
 {
+    Q_OBJECT
     friend class TransferHandler;
     friend class TransferTreeModel;
-
     public:
 
         /**
@@ -63,6 +66,17 @@ class KGET_EXPORT Transfer : public Job
             Tc_Selection      = 0x00010000
         };
 
+        enum Capability
+        {
+            Cap_SpeedLimit = 0x00000001,
+            Cap_MultipleMirrors = 0x00000002,
+            Cap_Resuming = 0x00000004,
+            Cap_Renaming = 0x00000008,
+            Cap_Moving = 0x00000010,
+            Cap_FindFilesize = 0x00000020
+        };
+        Q_DECLARE_FLAGS(Capabilities, Capability)
+
         enum LogLevel
         {
             info,
@@ -83,9 +97,61 @@ class KGET_EXPORT Transfer : public Job
 
         virtual ~Transfer();
 
+        /**
+         * Returns the capabilities this Transfer supports
+         */
+        Capabilities capabilities() const;
+
+        /**
+         * This functions gets called whenever a Transfer gets created. As opposed
+         * to init(), this isn't a virtual function and is not meant to be used in
+         * transfer plugins
+         */
+        void create();
+        
+        /**
+         * This functions gets called whenever a Transfer is going to be deleted. As opposed
+         * to deinit(), this isn't a virtual function and is not meant to be used in
+         * transfer plugins
+         */
+        void destroy();
+        
+        /**
+         * This function is called after the creation of a Transfer
+         * In transfer plugins you can put here whatever needs to be initialized
+         * @note this function creates a NepomukHandler
+         */
+        virtual void init();
+
+        /**
+         * This function is called before the deletion of a Transfer
+         * In transfer plugins you can put here whatever needs to be deinitialized
+         */
+        virtual void deinit() {}        
+        
+        /**
+         * Tries to repair file
+         * @param file the file of a download that should be repaired,
+         * if not defined all files of a download are going to be repaird
+         * @return true if a repair started, false if it was not nescessary
+         */
+        virtual bool repair(const KUrl &file = KUrl()) {Q_UNUSED(file) return false;}
+
         const KUrl & source() const            {return m_source;}
         const KUrl & dest() const              {return m_dest;}
 
+        /**
+         * @returns the directory the Transfer will be stored to
+         */
+        virtual KUrl directory() const {return m_dest.upUrl();}
+
+        /**
+         * Move the download to the new destination
+         * @param newDirectory is a directory where the download should be stored
+         * @returns true if newDestination can be used
+         */
+        virtual bool setDirectory(const KUrl &newDirectory);
+
         //Transfer status
         KIO::filesize_t totalSize() const      {return m_totalSize;}
         KIO::filesize_t downloadedSize() const {return m_downloadedSize;}
@@ -93,13 +159,32 @@ class KGET_EXPORT Transfer : public Job
         QString statusText() const             {return m_statusText;}
         QPixmap statusPixmap() const           {return m_statusPixmap;}
 
+        static QString statusText(Job::Status status);
+        static QPixmap statusPixmap(Job::Status status);
+
         int percent() const                    {return m_percent;}
         int downloadSpeed() const              {return m_downloadSpeed;}
+        int averageDownloadSpeed() const;
         int uploadSpeed() const                {return m_uploadSpeed;}
         virtual int remainingTime() const      {return KIO::calculateRemainingSeconds(totalSize(), downloadedSize(), downloadSpeed());}
         virtual int elapsedTime() const;
+        virtual bool isStalled() const         {return (status() == Job::Running && downloadSpeed() == 0);}
+        virtual bool isWorking() const         {return downloadSpeed() > 0;}
 
-        virtual bool supportsSpeedLimits() const {return false;}
+        /**
+         * The mirrors that are available
+         * bool if it is used, int how many paralell connections are allowed
+         * to the mirror
+         * @param file the file for which the availableMirrors should be get
+         */
+        virtual QHash<KUrl, QPair<bool, int> > availableMirrors(const KUrl &file) const;
+
+        /**
+         * Set the mirrors, int the number of paralell connections to the mirror
+         * bool if the mirror should be used
+         * @param file the file for which the availableMirrors should be set
+         */
+        virtual void setAvailableMirrors(const KUrl &file, const QHash<KUrl, QPair<bool, int> > &mirrors) {Q_UNUSED(file) Q_UNUSED(mirrors)}
 
         /**
          * Set the Transfer's UploadLimit
@@ -141,11 +226,6 @@ class KGET_EXPORT Transfer : public Job
          */
         void checkShareRatio();
 
-        // --- Job virtual functions ---
-        virtual void setDelay(int seconds);
-        virtual void delayTimerEvent();
-
-
         bool isSelected() const             {return m_isSelected;}
 
         /**
@@ -183,36 +263,67 @@ class KGET_EXPORT Transfer : public Job
          */
         TransferFactory * factory() const   {return m_factory;}
 
+        /**
+         * @returns a pointer to the FileModel containing all files of this download
+         */
+        virtual FileModel * fileModel() {return 0;}
+
+        /**
+         * @param file for which to get the verifier
+         * @return Verifier that allows you to add checksums manually verify a file etc.
+         */
+        virtual Verifier * verifier(const KUrl &file) {Q_UNUSED(file) return 0;}
+
+        /**
+         * @param file for which to get the signature
+         * @return Signature that allows you to add signatures and verify them
+         */
+        virtual Signature * signature(const KUrl &file) {Q_UNUSED(file) return 0;}
+
 #ifdef HAVE_NEPOMUK
         /**
          * Sets the NepomukHandler for the transfer
          * @param handler the new NepomukHandler
          */
-        void setNepomukHandler(NepomukHandler *handler) {m_nepomukHandler = handler;}
+        void setNepomukHandler(NepomukHandler *handler);
 
         /**
          * @returns a pointer to the NepomukHandler of this transfer
          */
-        NepomukHandler * nepomukHandler() const {return m_nepomukHandler;}
+        NepomukHandler * nepomukHandler() {return m_nepomukHandler;}
 #endif
 
         /**
          * Saves this transfer to the given QDomNode
          *
-         * @param n The pointer to the QDomNode where the transfer will be saved
-         * @return  The created QDomElement
+         * @param element The pointer to the QDomNode where the transfer will be saved
          */
         virtual void save(const QDomElement &element);
 
-    protected:
-        //Function used to load and save the transfer's info from xml
-        virtual void load(const QDomElement &e);
+        /**
+        * Loads the transfer's info from the QDomElement
+        *
+        * @param element The pointer to the QDomNode where info will be loaded from
+        */
+        virtual void load(const QDomElement *element);
 
+    signals:
+        /**
+         * Emitted when the capabilities of the Transfer change
+         */
+        void capabilitiesChanged();
+
+    protected:
         /**
          * Sets the Job status to jobStatus, the status text to text and
          * the status pixmap to pix.
          */
-        void setStatus(Job::Status jobStatus, const QString &text, const QPixmap &pix);
+        void setStatus(Job::Status jobStatus, const QString &text = QString(), const QPixmap &pix = QPixmap());
+
+        /**
+         * Sets the capabilities and automatically emits capabilitiesChanged
+         */
+        void setCapabilities(Capabilities capabilities);
 
         /**
          * Makes the TransferHandler associated with this transfer know that
@@ -245,6 +356,7 @@ class KGET_EXPORT Transfer : public Job
         bool m_isSelected;
 
     private:
+        Capabilities m_capabilities;
         int m_visibleUploadLimit;
         int m_visibleDownloadLimit;
         int m_runningSeconds;
@@ -261,4 +373,6 @@ class KGET_EXPORT Transfer : public Job
 #endif
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Transfer::Capabilities)
+
 #endif
