@@ -193,7 +193,7 @@ void TransferHistoryItemDelegate::slotDownload()
 {
    const QAbstractItemModel *model = static_cast <const QAbstractItemModel *> (m_selectedIndex.model());
 
-    NewTransferDialog::instance()->showDialog(model->data(m_selectedIndex,
+    NewTransferDialog::showNewTransferDialog(model->data(m_selectedIndex,
                                                     TransferHistoryCategorizedDelegate::RoleUrl).toString());
 }
 
