@@ -16,16 +16,14 @@
 */
 
 #include "historylogger.h"
-#include "historyconfig.h"
 
-#include <QRegExp>
-#include <QFile>
-#include <QDir>
-#include <QDateTime>
-#include <QtXml> // old qdom.h
-#include <QTimer>
-#include <QTextStream>
-#include <QList>
+#include <QtCore/QRegExp>
+#include <QtCore/QFile>
+#include <QtCore/QDir>
+#include <QtCore/QDateTime>
+#include <QtCore/QTimer>
+#include <QtCore/QTextStream>
+#include <QtCore/QList>
 #include <QtCore/QDate>
 
 #include <kdebug.h>
@@ -39,8 +37,16 @@
 #include "kopetemetacontact.h"
 #include "kopetechatsession.h"
 
+#include "historyconfig.h"
+
 bool messageTimestampLessThan(const Kopete::Message &m1, const Kopete::Message &m2)
 {
+	const Kopete::Contact* c1 = (m1.direction() == Kopete::Message::Outbound) ? m1.to().value(0) : m1.from();
+	const Kopete::Contact* c2 = (m2.direction() == Kopete::Message::Outbound) ? m2.to().value(0) : m2.from();
+
+	if (c1 == c2) // Messages from the same account, keep order as it was saved.
+		return false;
+
 	return m1.timestamp() < m2.timestamp();
 }
 
@@ -255,7 +261,9 @@ void HistoryLogger::appendMessage( const Kopete::Message &msg , const Kopete::Co
 		return;
 	}
 
-	QDomDocument doc=getDocument(c,0);
+	QDate date = msg.timestamp().date();
+		
+	QDomDocument doc=getDocument(c, QDate::currentDate().month() - date.month() - (QDate::currentDate().year() - date.year()) * 12);
 	QDomElement docElem = doc.documentElement();
 
 	if(docElem.isNull())
@@ -266,8 +274,8 @@ void HistoryLogger::appendMessage( const Kopete::Message &msg , const Kopete::Co
 		QDomElement headElem = doc.createElement( "head" );
 		docElem.appendChild( headElem );
 		QDomElement dateElem = doc.createElement( "date" );
-		dateElem.setAttribute( "year",  QString::number(QDate::currentDate().year()) );
-		dateElem.setAttribute( "month", QString::number(QDate::currentDate().month()) );
+		dateElem.setAttribute( "year",  QString::number(date.year()) );
+		dateElem.setAttribute( "month", QString::number(date.month()) );
 		headElem.appendChild(dateElem);
 		QDomElement myselfElem = doc.createElement( "contact" );
 		myselfElem.setAttribute( "type",  "myself" );
@@ -293,7 +301,7 @@ void HistoryLogger::appendMessage( const Kopete::Message &msg , const Kopete::Co
 	// On hight-traffic channel, saving can take lots of CPU. (because the file is big)
 	// So i wait a time proportional to the time needed to save..
 
-	const QString filename=getFileName(c,QDate::currentDate());
+	const QString filename=getFileName(c, date);
 	if(!m_toSaveFileName.isEmpty() && m_toSaveFileName != filename)
 	{ //that mean the contact or the month has changed, save it now.
 		saveToDisk();
@@ -380,10 +388,10 @@ QList<Kopete::Message> HistoryLogger::readMessages(QDate date)
 				{ //parse only if we don't hide it
 
 					QString f=msgElem2.attribute("from" );
-					const Kopete::Contact *from=f.isNull()? 0L : contact->account()->contacts()[f];
+					const Kopete::Contact *from=f.isNull()? 0L : contact->account()->contacts().value( f );
 
 					if(!from)
-						from= dir==Kopete::Message::Inbound ? contact : contact->account()->myself();
+						from = (dir == Kopete::Message::Inbound) ? contact : contact->account()->myself();
 
 					Kopete::ContactPtrList to;
 					to.append( dir==Kopete::Message::Inbound ? contact->account()->myself() : contact );
@@ -404,7 +412,23 @@ QList<Kopete::Message> HistoryLogger::readMessages(QDate date)
 
 	}
 
-	qSort(messages.begin(), messages.end(), messageTimestampLessThan);
+	//Bubble Sort, can't use qStableSort because we have to compare surrounding items only, mostly
+	//it will be only O(n) sort, because we will only have one contact in metacontact for a day.
+	const int size = messages.size();
+	for (int i = 0; i < size; i++)
+	{
+		bool swap = false;
+		for (int j = 0; j < size - 1; j++)
+		{
+			if (messageTimestampLessThan(messages.at(j + 1), messages.at(j))) {
+				messages.swap(j, j + 1);
+				swap = true;
+			}
+		}
+
+		if (!swap)
+			break;
+	}
 	return messages;
 }
 
@@ -584,10 +608,10 @@ QList<Kopete::Message> HistoryLogger::readMessages(int lines,
 				{
 					Q_ASSERT(currentContact);
 					QString f=msgElem.attribute("from" );
-					const Kopete::Contact *from=f.isNull() ? 0L : currentContact->account()->contacts()[f];
+					const Kopete::Contact *from=f.isNull() ? 0L : currentContact->account()->contacts().value(f);
 
-					if(!from)
-						from= dir==Kopete::Message::Inbound ? currentContact : currentContact->account()->myself();
+					if( !from )
+						from = (dir == Kopete::Message::Inbound) ? currentContact : currentContact->account()->myself();
 
 					Kopete::ContactPtrList to;
 					to.append( dir==Kopete::Message::Inbound ? currentContact->account()->myself() : const_cast<Kopete::Contact*>(currentContact) );
