@@ -25,8 +25,8 @@
 
 using namespace GroupWise;
 
-EventProtocol::EventProtocol(QObject *parent, const char *name)
- : InputProtocolBase(parent, name)
+EventProtocol::EventProtocol(QObject *parent)
+ : InputProtocolBase(parent)
 {
 }
 
@@ -34,15 +34,15 @@ EventProtocol::~EventProtocol()
 {
 }
 
-Transfer * EventProtocol::parse( const QByteArray & wire, uint& bytes )
+Transfer * EventProtocol::parse( QByteArray & wire, uint& bytes )
 {
 	m_bytes = 0;
-	//m_din = new QDataStream( wire, IO_ReadOnly );
-	QBuffer inBuf( wire );
-	inBuf.open( IO_ReadOnly); 
+	//m_din = new QDataStream( wire, QIODevice::ReadOnly );
+	QBuffer inBuf( &wire );
+	inBuf.open( QIODevice::ReadOnly); 
 	m_din.setDevice( &inBuf );
 	m_din.setByteOrder( QDataStream::LittleEndian );
-	Q_UINT32 type;
+	quint32 type;
 
 	if ( !okToProceed() )
 	{
@@ -51,7 +51,7 @@ Transfer * EventProtocol::parse( const QByteArray & wire, uint& bytes )
 	}
 	// read the event type
 	m_din >> type;
-	m_bytes += sizeof( Q_UINT32 );
+	m_bytes += sizeof( quint32 );
 	
 	debug( QString( "EventProtocol::parse() Reading event of type %1" ).arg( type ) );
 	if ( type > Stop )
@@ -71,14 +71,14 @@ Transfer * EventProtocol::parse( const QByteArray & wire, uint& bytes )
 	
 	// now create an event object
 	//HACK: lowercased DN
-	EventTransfer * tentative = new EventTransfer( type, source.lower(), QDateTime::currentDateTime() );
+	EventTransfer * tentative = new EventTransfer( type, source.toLower(), QDateTime::currentDateTime() );
 
 	// add any additional data depending on the type of event
 	// Note: if there are any errors in the way the data is read below, we will soon be OutOfSync
 	QString statusText;
 	QString guid;
-	Q_UINT16 status;
-	Q_UINT32 flags;
+	quint16 status;
+	quint32 flags;
 	QString message;
 	
 	switch ( type )
@@ -90,7 +90,7 @@ Transfer * EventProtocol::parse( const QByteArray & wire, uint& bytes )
 				return 0;
 			}
 			m_din >> status;
-			m_bytes += sizeof( Q_UINT16 );
+			m_bytes += sizeof( quint16 );
 			if ( !readString( statusText ) )
 			{
 				m_din.unsetDevice();
@@ -202,12 +202,12 @@ Transfer * EventProtocol::parse( const QByteArray & wire, uint& bytes )
 	return tentative;
 }
 
-bool EventProtocol::readFlags( Q_UINT32 &flags)
+bool EventProtocol::readFlags( quint32 &flags)
 {
 	if ( okToProceed() )
 	{
 		m_din >> flags;
-		m_bytes += sizeof( Q_UINT32 );
+		m_bytes += sizeof( quint32 );
 		return true;
 	}
 	return false;
