@@ -84,6 +84,14 @@ public:
 	/** Set the server port */
 	void setServerPort( int port );
 
+    /** Set the proxy server address */
+    void setProxyServerAddress( const QString& server );
+
+    /** Set the proxy server port */
+    void setProxyServerPort( int port );
+
+    void setProxyServerEnabled(bool);
+
 	/** Returns codec for account's default encoding */
 	QTextCodec* defaultCodec() const;
 
