@@ -2,7 +2,7 @@
  *
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: pppstats.h 753586 2007-12-27 22:28:45Z uwolfer $
+ * $Id: pppstats.h 870811 2008-10-13 10:32:15Z hasso $
  *
  * (c) 1997-1998 Bernd Johannes Wuebben <wuebben@kde.org>
  * (c) 1997-1999 Mario Weilguni <mweilguni@kde.org>
@@ -33,10 +33,18 @@
 #ifdef Q_OS_BSD4
 #include <sys/types.h>
 #include <sys/socket.h>
+#ifdef __DragonFly__
+#include <net/ppp_layer/ppp_defs.h>
+#else
 #include <net/ppp_defs.h>
+#endif
 #include <net/if.h>
+#ifdef __DragonFly__
+#include <net/ppp/if_ppp.h>
+#else
 #include <net/if_ppp.h>
 #endif
+#endif
 
 class QTimer;
 
