@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2007 - 2009 Urs Wolfer <uwolfer @ kde.org>
+** Copyright (C) 2007 - 2010 Urs Wolfer <uwolfer @ kde.org>
 ** Copyright (C) 2009 Tony Murray <murraytony @ gmail.com>
 **
 ** This file is part of KDE.
@@ -29,6 +29,7 @@
 #include "config/preferencesdialog.h"
 #include "floatingtoolbar.h"
 #include "bookmarkmanager.h"
+#include "connectiondelegate.h"
 #include "remotedesktopsmodel.h"
 #include "systemtrayicon.h"
 #include "tabbedviewwidget.h"
@@ -41,10 +42,10 @@
 #include <KAction>
 #include <KActionCollection>
 #include <KActionMenu>
-#include <KApplication>
 #include <KComboBox>
 #include <KEditToolBar>
 #include <KIcon>
+#include <KInputDialog>
 #include <KLineEdit>
 #include <KLocale>
 #include <KMenu>
@@ -53,42 +54,35 @@
 #include <KNotifyConfigWidget>
 #include <KPluginInfo>
 #include <KPushButton>
-#include <KShortcut>
 #include <KShortcutsDialog>
 #include <KStatusBar>
 #include <KToggleAction>
 #include <KToggleFullScreenAction>
-#include <KUrlComboBox>
 #include <KUrlNavigator>
 #include <KServiceTypeTrader>
-#include <KStandardDirs>
 
 #include <QClipboard>
-#include <QCloseEvent>
-#include <QDesktopWidget>
 #include <QDockWidget>
 #include <QFontMetrics>
+#include <QGroupBox>
 #include <QHeaderView>
 #include <QLabel>
 #include <QLayout>
 #include <QScrollArea>
 #include <QSortFilterProxyModel>
+#include <QTableView>
 #include <QTimer>
-#include <QToolButton>
-#include <QToolTip>
-#include <QTreeView>
+#include <QToolBar>
 
 MainWindow::MainWindow(QWidget *parent)
         : KXmlGuiWindow(parent),
         m_fullscreenWindow(0),
         m_addressNavigator(0),
         m_toolBar(0),
-        m_topBottomBorder(0),
-        m_leftRightBorder(0),
         m_currentRemoteView(-1),
         m_systemTrayIcon(0),
-        m_remoteDesktopsTreeView(0),
-        m_remoteDesktopsNewConnectionTabTreeView(0),
+        m_dockWidgetTableView(0),
+        m_newConnectionTableView(0),
 #ifdef TELEPATHY_SUPPORT
         m_tubesManager(0),
 #endif
@@ -290,22 +284,23 @@ void MainWindow::restoreOpenSessions()
     }
 }
 
-void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConnected)
+void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConnected, const QString &tabName)
 {
     m_switchFullscreenWhenConnected = switchFullscreenWhenConnected;
 
     const KUrl url = newUrl.isEmpty() ? m_addressNavigator->uncommittedUrl() : newUrl;
 
     if (!url.isValid() || (url.host().isEmpty() && url.port() < 0)
-            || !url.path().isEmpty()) {
+        || (!url.path().isEmpty() && url.path() != QLatin1String("/"))) {
         KMessageBox::error(this,
                            i18n("The entered address does not have the required form."),
                            i18n("Malformed URL"));
         return;
     }
 
-    if (m_addressNavigator)
-        m_addressNavigator->setUrl(KUrl(url.scheme().toLower() + "://"));
+    if (m_addressNavigator) {
+        m_addressNavigator->setLocationUrl(url);
+    }
 
     RemoteView *view = 0;
     KConfigGroup configGroup = Settings::self()->config()->group("hostpreferences").group(url.prettyUrl(KUrl::RemoveTrailingSlash));
@@ -327,7 +322,7 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
 
     // Configure the view
     HostPreferences* prefs = view->hostPreferences();
-    if (! prefs->showDialogIfNeeded()) {
+    if (! prefs->showDialogIfNeeded(this)) {
         delete view;
         return;
     }
@@ -337,7 +332,7 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
     view->setGrabAllKeys(prefs->grabAllKeys());
     view->showDotCursor(prefs->showLocalCursor() ? RemoteView::CursorOn : RemoteView::CursorOff);
     view->setViewOnly(prefs->viewOnly());
-    if (! switchFullscreenWhenConnected) view->enableScaling(prefs->windowedScale()); 
+    if (! switchFullscreenWhenConnected) view->enableScaling(prefs->windowedScale());
 
     connect(view, SIGNAL(framebufferSizeChanged(int, int)), this, SLOT(resizeTabWidget(int, int)));
     connect(view, SIGNAL(statusChanged(RemoteView::RemoteStatus)), this, SLOT(statusChanged(RemoteView::RemoteStatus)));
@@ -345,31 +340,25 @@ void MainWindow::newConnection(const KUrl &newUrl, bool switchFullscreenWhenConn
 
     m_remoteViewList.append(view);
 
-//     view->resize(0, 0);
- 
-    if (m_remoteViewList.size() == 1) {
-        kDebug(5010) << "First connection, restoring window size.";
-        restoreWindowSize(view->hostPreferences()->configGroup());
-    }
-
     QScrollArea *scrollArea = createScrollArea(m_tabWidget, view);
 
     const int indexOfNewConnectionWidget = m_tabWidget->indexOf(m_newConnectionWidget);
     if (indexOfNewConnectionWidget >= 0)
         m_tabWidget->removeTab(indexOfNewConnectionWidget);
 
-    const int newIndex = m_tabWidget->addTab(scrollArea, KIcon("krdc"), url.prettyUrl(KUrl::RemoveTrailingSlash));
+    const int newIndex = m_tabWidget->addTab(scrollArea, KIcon("krdc"), tabName.isEmpty() ? url.prettyUrl(KUrl::RemoveTrailingSlash) : tabName);
     m_tabWidget->setCurrentIndex(newIndex);
     tabChanged(newIndex); // force to update m_currentRemoteView (tabChanged is not emitted when start page has been disabled)
 
     view->start();
 }
 
-void MainWindow::openFromDockWidget(const QModelIndex &index)
+void MainWindow::openFromRemoteDesktopsModel(const QModelIndex &index)
 {
-    const QString data = index.data(10001).toString();
-    if (!data.isEmpty()) {
-        const KUrl url(data);
+    const QString urlString = index.data(10001).toString();
+    const QString nameString = index.data(10003).toString();
+    if (!urlString.isEmpty()) {
+        const KUrl url(urlString);
         // first check if url has already been opened; in case show the tab
         for (int i = 0; i < m_remoteViewList.count(); ++i) {
             if (m_remoteViewList.at(i)->url() == url) {
@@ -377,49 +366,47 @@ void MainWindow::openFromDockWidget(const QModelIndex &index)
                 return;
             }
         }
-        newConnection(url);
+        newConnection(url, false, nameString);
     }
 }
 
 void MainWindow::resizeTabWidget(int w, int h)
 {
-    kDebug(5010) << "tabwidget resize: w: " << w << ", h: " << h;
+    kDebug(5010) << "tabwidget resize, view size: w: " << w << ", h: " << h;
     if (m_fullscreenWindow) {
         kDebug(5010) << "in fullscreen mode, refusing to resize";
         return;
     }
 
-    RemoteView* view = m_currentRemoteView >= 0 ? m_remoteViewList.at(m_currentRemoteView) : 0;
-    if (view && view->scaling()) return;
+    const QSize viewSize = QSize(w,h);
+    QDesktopWidget *desktop = QApplication::desktop();
 
-    if (m_topBottomBorder == 0) { // the values are not cached yet
-        QScrollArea *tmp = qobject_cast<QScrollArea *>(m_tabWidget->currentWidget());
+    if (Settings::fullscreenOnConnect()) {
+        int currentScreen = desktop->screenNumber(this);
+        const QSize screenSize = desktop->screenGeometry(currentScreen).size();
 
-        m_leftRightBorder = m_tabWidget->width() - m_tabWidget->currentWidget()->width() + (2 * tmp->frameWidth());
-        m_topBottomBorder = m_tabWidget->height() - m_tabWidget->currentWidget()->height() + (2 * tmp->frameWidth());
-
-        kDebug(5010) << "tabwidget border: w: " << m_leftRightBorder << ", h: " << m_topBottomBorder;
+        if (screenSize == viewSize) {
+            kDebug(5010) << "screen size equal to target view size -> switch to fullscreen mode";
+            switchFullscreen();
+            return;
+        }
     }
 
-    const int newTabWidth = w + m_leftRightBorder;
-    const int newTabHeight = h + m_topBottomBorder;
+    if (Settings::resizeOnConnect()) {
+        QWidget* currentWidget = m_tabWidget->currentWidget();
+        const QSize newWindowSize = size() - currentWidget->frameSize() + viewSize;
 
-    const QSize newWindowSize = size() - m_tabWidget->size() + QSize(newTabWidth, newTabHeight);
+        const QSize desktopSize = desktop->availableGeometry().size();
+        kDebug(5010) << "new window size: " << newWindowSize << " available space:" << desktopSize;
 
-    const QSize desktopSize = QSize(QApplication::desktop()->availableGeometry().width(),
-                              QApplication::desktop()->availableGeometry().height());
+        if ((newWindowSize.width() >= desktopSize.width()) || (newWindowSize.height() >= desktopSize.height())) {
+            kDebug(5010) << "remote desktop needs more space than available -> show window maximized";
+            setWindowState(windowState() | Qt::WindowMaximized);
+            return;
+        }
 
-    if ((newWindowSize.height() >= desktopSize.height()) || (newWindowSize.width() >= desktopSize.width())) {
-        kDebug(5010) << "remote desktop needs more place than available -> show window maximized";
-        setWindowState(windowState() | Qt::WindowMaximized);
-        return;
+        resize(newWindowSize);
     }
-
-    //WORKAROUND: QTabWidget resize problem. Let's see if there is a clean solution for this issue.
-    m_tabWidget->setMinimumSize(newTabWidth, newTabHeight);
-    m_tabWidget->adjustSize();
-    QCoreApplication::processEvents();
-    m_tabWidget->setMinimumSize(500, 400);
 }
 
 void MainWindow::statusChanged(RemoteView::RemoteStatus status)
@@ -458,7 +445,9 @@ void MainWindow::statusChanged(RemoteView::RemoteStatus status)
             switchFullscreen();
         }
 
-        m_bookmarkManager->addHistoryBookmark();
+        if (Settings::rememberHistory()) {
+            m_bookmarkManager->addHistoryBookmark();
+        }
 
         break;
     default:
@@ -494,7 +483,7 @@ void MainWindow::switchFullscreen()
         restoreGeometry(m_mainWindowGeometry);
         if (m_systemTrayIcon) m_systemTrayIcon->setAssociatedWidget(this);
 
-        foreach (RemoteView *currentView, m_remoteViewList) {
+        foreach(RemoteView *currentView, m_remoteViewList) {
             currentView->enableScaling(currentView->hostPreferences()->windowedScale());
         }
 
@@ -514,19 +503,19 @@ void MainWindow::switchFullscreen()
         // Entering full screen mode
         m_fullscreenWindow = new QWidget(this, Qt::Window);
         m_fullscreenWindow->setWindowTitle(i18nc("window title when in full screen mode (for example displayed in tasklist)",
-                                                 "KDE Remote Desktop Client (Full Screen)"));
+                                           "KDE Remote Desktop Client (Full Screen)"));
 
         m_mainWindowGeometry = saveGeometry();
 
         m_tabWidget->setTabBarHidden(true);
         m_tabWidget->setDocumentMode(true);
 
-        foreach (RemoteView *currentView, m_remoteViewList) {
+        foreach(RemoteView *currentView, m_remoteViewList) {
             currentView->enableScaling(currentView->hostPreferences()->fullscreenScale());
         }
 
         QVBoxLayout *fullscreenLayout = new QVBoxLayout(m_fullscreenWindow);
-        fullscreenLayout->setMargin(0);
+        fullscreenLayout->setContentsMargins(QMargins(0, 0, 0, 0));
         fullscreenLayout->addWidget(m_tabWidget);
 
         KToggleFullScreenAction::setFullScreen(m_fullscreenWindow, true);
@@ -605,7 +594,7 @@ void MainWindow::closeTab(QWidget *widget)
     kDebug(5010) << index;
 
     if (!isNewConnectionPage) {
-        RemoteView *view = m_remoteViewList.takeAt(m_currentRemoteView);
+        RemoteView *view = m_remoteViewList.takeAt(index);
         view->startQuitting();
 #ifdef TELEPATHY_SUPPORT
         m_tubesManager->closeTube(view->url());
@@ -613,7 +602,7 @@ void MainWindow::closeTab(QWidget *widget)
         widget->deleteLater();
     }
 
-    m_tabWidget->removeTab(index);
+    m_tabWidget->removePage(widget);
 
     // if closing the last connection, create new connection tab
     if (m_tabWidget->count() == 0) {
@@ -636,6 +625,11 @@ void MainWindow::openTabSettings(QWidget *widget)
     const QString url = view->url().url();
     kDebug(5010) << url;
 
+    showSettingsDialog(url);
+}
+
+void MainWindow::showSettingsDialog(const QString &url)
+{
     HostPreferences *prefs = 0;
 
     foreach(RemoteViewFactory *factory, remoteViewFactoriesList()) {
@@ -651,8 +645,7 @@ void MainWindow::openTabSettings(QWidget *widget)
 
     if (prefs) {
         prefs->setShownWhileConnected(true);
-        prefs->showDialog();
-        delete prefs;
+        prefs->showDialog(this);
     } else {
         KMessageBox::error(this,
                            i18n("The selected host cannot be handled."),
@@ -660,6 +653,59 @@ void MainWindow::openTabSettings(QWidget *widget)
     }
 }
 
+void MainWindow::showConnectionContextMenu(const QPoint &pos)
+{
+    // QTableView does not take headers into account when it does mapToGlobal(), so calculate the offset
+    QPoint offset = QPoint(m_newConnectionTableView->verticalHeader()->size().width(),
+                           m_newConnectionTableView->horizontalHeader()->size().height());
+    QModelIndex index = m_newConnectionTableView->indexAt(pos);
+
+    if (!index.isValid())
+        return;
+
+    const QString url = index.data(10001).toString();
+    const QString title = m_remoteDesktopsModelProxy->index(index.row(), RemoteDesktopsModel::Title).data(Qt::DisplayRole).toString();
+    const QString source = m_remoteDesktopsModelProxy->index(index.row(), RemoteDesktopsModel::Source).data(Qt::DisplayRole).toString();
+
+    KMenu *menu = new KMenu(m_newConnectionTableView);
+    menu->addTitle(url);
+
+    QAction *connectAction = menu->addAction(KIcon("network-connect"), i18n("Connect"));
+    QAction *renameAction = menu->addAction(KIcon("edit-rename"), i18n("Rename"));
+    QAction *settingsAction = menu->addAction(KIcon("configure"), i18n("Settings"));
+    QAction *deleteAction = menu->addAction(KIcon("edit-delete"), i18n("Delete"));
+
+    // not very clean, but it works,
+    if (!(source == i18nc("Where each displayed link comes from", "Bookmarks") ||
+            source == i18nc("Where each displayed link comes from", "History"))) {
+        renameAction->setEnabled(false);
+        deleteAction->setEnabled(false);
+    }
+
+    QAction *selectedAction = menu->exec(m_newConnectionTableView->mapToGlobal(pos + offset));
+
+    if (selectedAction == connectAction) {
+        openFromRemoteDesktopsModel(index);
+    } else if (selectedAction == renameAction) {
+        //TODO: use inline editor if possible
+        bool ok = false;
+        const QString newTitle = KInputDialog::getText(i18n("Rename %1", title), i18n("Rename %1 to", title), "", &ok, this);
+        if (ok && !newTitle.isEmpty()) {
+            BookmarkManager::updateTitle(m_bookmarkManager->getManager(), url, newTitle);
+        }
+    } else if (selectedAction == settingsAction) {
+        showSettingsDialog(url);
+    } else if (selectedAction == deleteAction) {
+
+        if (KMessageBox::warningContinueCancel(this, i18n("Are you sure you want to delete %1?", url), i18n("Delete %1", title),
+                                               KStandardGuiItem::del()) == KMessageBox::Continue) {
+            BookmarkManager::removeByUrl(m_bookmarkManager->getManager(), url);
+        }
+    }
+
+    menu->deleteLater();
+}
+
 void MainWindow::tabContextMenu(QWidget *widget, const QPoint &point)
 {
     RemoteViewScrollArea *scrollArea = qobject_cast<RemoteViewScrollArea*>(widget);
@@ -721,11 +767,11 @@ void MainWindow::scale(bool scale)
 
     RemoteView* view = m_remoteViewList.at(m_currentRemoteView);
     view->enableScaling(scale);
-    if (m_fullscreenWindow) 
+    if (m_fullscreenWindow)
         view->hostPreferences()->setFullscreenScale(scale);
     else
         view->hostPreferences()->setWindowedScale(scale);
-    
+
     saveHostPrefs();
 }
 
@@ -751,34 +797,38 @@ void MainWindow::showRemoteViewToolbar()
         connect(m_tabWidget, SIGNAL(currentChanged(int)), sessionComboBox, SLOT(setCurrentIndex(int)));
         m_toolBar->addWidget(sessionComboBox);
 
-        m_toolBar->addAction(actionCollection()->action("new_connection"));
-        m_toolBar->addAction(actionCollection()->action("switch_fullscreen"));
+        QToolBar *buttonBox = new QToolBar(m_toolBar);
+
+        buttonBox->addAction(actionCollection()->action("new_connection"));
+        buttonBox->addAction(actionCollection()->action("switch_fullscreen"));
 
         QAction *minimizeAction = new QAction(m_toolBar);
         minimizeAction->setIcon(KIcon("go-down"));
         minimizeAction->setText(i18n("Minimize Full Screen Window"));
         connect(minimizeAction, SIGNAL(triggered()), m_fullscreenWindow, SLOT(showMinimized()));
-        m_toolBar->addAction(minimizeAction);
+        buttonBox->addAction(minimizeAction);
 
-        m_toolBar->addAction(actionCollection()->action("take_screenshot"));
-        m_toolBar->addAction(actionCollection()->action("view_only"));
-        m_toolBar->addAction(actionCollection()->action("show_local_cursor"));
-        m_toolBar->addAction(actionCollection()->action("grab_all_keys"));
-        m_toolBar->addAction(actionCollection()->action("scale"));
-        m_toolBar->addAction(actionCollection()->action("disconnect"));
+        buttonBox->addAction(actionCollection()->action("take_screenshot"));
+        buttonBox->addAction(actionCollection()->action("view_only"));
+        buttonBox->addAction(actionCollection()->action("show_local_cursor"));
+        buttonBox->addAction(actionCollection()->action("grab_all_keys"));
+        buttonBox->addAction(actionCollection()->action("scale"));
+        buttonBox->addAction(actionCollection()->action("disconnect"));
 
         QAction *stickToolBarAction = new QAction(m_toolBar);
         stickToolBarAction->setCheckable(true);
         stickToolBarAction->setIcon(KIcon("object-locked"));
         stickToolBarAction->setText(i18n("Stick Toolbar"));
         connect(stickToolBarAction, SIGNAL(triggered(bool)), m_toolBar, SLOT(setSticky(bool)));
-        m_toolBar->addAction(stickToolBarAction);
+        buttonBox->addAction(stickToolBarAction);
+
+        m_toolBar->addWidget(buttonBox);
     }
 
     m_toolBar->showAndAnimate();
 }
 
-void setActionStatus(QAction* action, bool enabled, bool visible, bool checked) 
+void setActionStatus(QAction* action, bool enabled, bool visible, bool checked)
 {
     action->setEnabled(enabled);
     action->setVisible(visible);
@@ -800,12 +850,12 @@ void MainWindow::updateActionStatus()
     actionCollection()->action("disconnect")->setEnabled(enabled);
 
     setActionStatus(actionCollection()->action("view_only"),
-                    enabled, 
+                    enabled,
                     true,
                     view ? view->viewOnly() : false);
 
     setActionStatus(actionCollection()->action("show_local_cursor"),
-                    enabled, 
+                    enabled,
                     view ? view->supportsLocalCursor() : false,
                     view ? view->dotCursorState() == RemoteView::CursorOn : false);
 
@@ -887,10 +937,10 @@ void MainWindow::quit(bool systemEvent)
 {
     const bool haveRemoteConnections = !m_remoteViewList.isEmpty();
     if (systemEvent || !haveRemoteConnections || KMessageBox::warningContinueCancel(this,
-                                           i18n("Are you sure you want to quit the KDE Remote Desktop Client?"),
-                                           i18n("Confirm Quit"),
-                                           KStandardGuiItem::quit(), KStandardGuiItem::cancel(),
-                                           "DoNotAskBeforeExit") == KMessageBox::Continue) {
+            i18n("Are you sure you want to quit the KDE Remote Desktop Client?"),
+            i18n("Confirm Quit"),
+            KStandardGuiItem::quit(), KStandardGuiItem::cancel(),
+            "DoNotAskBeforeExit") == KMessageBox::Continue) {
 
         if (Settings::rememberSessions()) { // remember open remote views for next startup
             QStringList list;
@@ -971,8 +1021,13 @@ void MainWindow::saveHostPrefs(RemoteView* view)
     if (!view)
         view = m_currentRemoteView >= 0 ? m_remoteViewList.at(m_currentRemoteView) : 0;
 
-    if (view)
-        saveWindowSize(view->hostPreferences()->configGroup());
+    // should saving this be a user option?
+    if (view && view->scaling()) {
+        QSize viewSize = m_tabWidget->currentWidget()->size();
+        kDebug(5010) << "saving window size:" << viewSize;
+        view->hostPreferences()->setWidth(viewSize.width());
+        view->hostPreferences()->setHeight(viewSize.height());
+    }
 
     Settings::self()->config()->sync();
 }
@@ -1002,13 +1057,13 @@ QWidget* MainWindow::newConnectionWidget()
     m_newConnectionWidget = new QWidget(this);
 
     QVBoxLayout *startLayout = new QVBoxLayout(m_newConnectionWidget);
-    startLayout->setMargin(20);
+    startLayout->setContentsMargins(QMargins(8, 12, 8, 4));
 
-    QLabel *headerLabel = new QLabel(this);
-    headerLabel->setText(i18n("<h1>KDE Remote Desktop Client</h1><br /><br />What would you like to do?<br />"));
+    QLabel *headerLabel = new QLabel(m_newConnectionWidget);
+    headerLabel->setText(i18n("<h1>KDE Remote Desktop Client</h1><br />Enter or select the address of the desktop you would like to connect to."));
 
-    QLabel *headerIconLabel = new QLabel(this);
-    headerIconLabel->setPixmap(KIcon("krdc").pixmap(128));
+    QLabel *headerIconLabel = new QLabel(m_newConnectionWidget);
+    headerIconLabel->setPixmap(KIcon("krdc").pixmap(80));
 
     QHBoxLayout *headerLayout = new QHBoxLayout;
     headerLayout->addWidget(headerLabel, 1, Qt::AlignTop);
@@ -1018,7 +1073,10 @@ QWidget* MainWindow::newConnectionWidget()
     {
         QHBoxLayout *connectLayout = new QHBoxLayout;
         const QString initialProtocol(!m_remoteViewFactories.isEmpty() ? (*m_remoteViewFactories.begin())->scheme() : QString());
-        m_addressNavigator = new KUrlNavigator(0, KUrl(initialProtocol + "://"), this);
+        KUrl url;
+        url.setProtocol(initialProtocol);
+        url.setPath(QLatin1String("//"));
+        m_addressNavigator = new KUrlNavigator(0, url, m_newConnectionWidget);
 
         QStringList schemes;
         foreach(RemoteViewFactory *factory, m_remoteViewFactories) {
@@ -1029,10 +1087,11 @@ QWidget* MainWindow::newConnectionWidget()
         m_addressNavigator->setUrlEditable(Settings::normalUrlInputLine());
         connect(m_addressNavigator, SIGNAL(returnPressed()), SLOT(newConnection()));
         m_addressNavigator->setFocus();
+        m_addressNavigator->setToolTip(i18n("Type an IP or DNS Name here. Clear the line to get a list of connection methods."));
 
-        QLabel *addressLabel = new QLabel(i18n("Connect to:"), this);
+        QLabel *addressLabel = new QLabel(i18n("Connect to:"), m_newConnectionWidget);
 
-        KPushButton *connectButton = new KPushButton(this);
+        KPushButton *connectButton = new KPushButton(m_newConnectionWidget);
         connectButton->setToolTip(i18n("Goto Address"));
         connectButton->setIcon(KIcon("go-jump-locationbar"));
         connect(connectButton, SIGNAL(clicked()), SLOT(newConnection()));
@@ -1040,35 +1099,61 @@ QWidget* MainWindow::newConnectionWidget()
         connectLayout->addWidget(addressLabel);
         connectLayout->addWidget(m_addressNavigator, 1);
         connectLayout->addWidget(connectButton);
+        connectLayout->setContentsMargins(QMargins(0, 6, 0, 10));
         startLayout->addLayout(connectLayout);
     }
 
     {
-        QWidget *remoteDesktopsDockLayoutWidget = new QWidget(this);
-        QVBoxLayout *remoteDesktopsDockLayout = new QVBoxLayout(remoteDesktopsDockLayoutWidget);
-        remoteDesktopsDockLayout->setMargin(0);
-        m_remoteDesktopsNewConnectionTabTreeView = new QTreeView(remoteDesktopsDockLayoutWidget);
+        QGroupBox *remoteDesktopsGroupWidget = new QGroupBox(m_newConnectionWidget);
+        QVBoxLayout *remoteDesktopsGroupLayout = new QVBoxLayout(remoteDesktopsGroupWidget);
+        QHBoxLayout *filterLayout = new QHBoxLayout;
+        m_newConnectionTableView = new QTableView(remoteDesktopsGroupWidget);
+        m_newConnectionTableView->setModel(m_remoteDesktopsModelProxy);
 
-        m_remoteDesktopsNewConnectionTabTreeView->setModel(m_remoteDesktopsModelProxy);
-        m_remoteDesktopsNewConnectionTabTreeView->header()->hide();
-        m_remoteDesktopsNewConnectionTabTreeView->expandAll();
-        m_remoteDesktopsNewConnectionTabTreeView->setItemsExpandable(true);
-        m_remoteDesktopsNewConnectionTabTreeView->setRootIsDecorated(false);
-        connect(m_remoteDesktopsNewConnectionTabTreeView, SIGNAL(doubleClicked(const QModelIndex &)),
-                                        SLOT(openFromDockWidget(const QModelIndex &)));
+        // set up the view so it looks nice
+        m_newConnectionTableView->setItemDelegate(new ConnectionDelegate(m_newConnectionTableView));
+        m_newConnectionTableView->setShowGrid(false);
+        m_newConnectionTableView->verticalHeader()->hide();
+        m_newConnectionTableView->verticalHeader()->setDefaultSectionSize(
+            m_newConnectionTableView->fontMetrics().height() + 3);
+        m_newConnectionTableView->horizontalHeader()->setStretchLastSection(true);
+//         m_newConnectionTableView->resizeColumnsToContents(); // not needed when resize mode is ResizeToContents
+        m_newConnectionTableView->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
+        m_newConnectionTableView->setAlternatingRowColors(true);
 
-        KLineEdit *filterLineEdit = new KLineEdit(remoteDesktopsDockLayoutWidget);
-        filterLineEdit->setClickMessage(i18n("Filter"));
+        // set up sorting and actions (double click open, right click custom menu)
+        m_newConnectionTableView->setSortingEnabled(true);
+        m_newConnectionTableView->sortByColumn(Settings::connectionListSortColumn(), Qt::SortOrder(Settings::connectionListSortOrder()));
+        connect(m_newConnectionTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(const int, const Qt::SortOrder)),
+                SLOT(saveConnectionListSort(const int, const Qt::SortOrder)));
+        connect(m_newConnectionTableView, SIGNAL(doubleClicked(const QModelIndex &)),
+                SLOT(openFromRemoteDesktopsModel(const QModelIndex &)));
+        m_newConnectionTableView->setContextMenuPolicy(Qt::CustomContextMenu);
+        connect(m_newConnectionTableView, SIGNAL(customContextMenuRequested(QPoint)), SLOT(showConnectionContextMenu(QPoint)));
+
+        // set up the filter input field
+        QLabel *filterLabel = new QLabel(i18nc("Verb, to remove items that don't match", "Filter"), remoteDesktopsGroupWidget);
+        KLineEdit *filterLineEdit = new KLineEdit(remoteDesktopsGroupWidget);
+        filterLineEdit->setClickMessage(i18n("Type here to filter the connection list."));
         filterLineEdit->setClearButtonShown(true);
         connect(filterLineEdit, SIGNAL(textChanged(const QString &)), SLOT(updateFilter(const QString &)));
-        remoteDesktopsDockLayout->addWidget(filterLineEdit);
-        remoteDesktopsDockLayout->addWidget(m_remoteDesktopsNewConnectionTabTreeView);
-        startLayout->addWidget(remoteDesktopsDockLayoutWidget);
+        filterLayout->addWidget(filterLabel);
+        filterLayout->addWidget(filterLineEdit);
+
+        remoteDesktopsGroupLayout->addLayout(filterLayout);
+        remoteDesktopsGroupLayout->addWidget(m_newConnectionTableView);
+        startLayout->addWidget(remoteDesktopsGroupWidget);
     }
 
     return m_newConnectionWidget;
 }
 
+void MainWindow::saveConnectionListSort(const int logicalindex, const Qt::SortOrder order)
+{
+    Settings::setConnectionListSortColumn(logicalindex);
+    Settings::setConnectionListSortOrder(order);
+    Settings::self()->writeConfig();
+}
 
 void MainWindow::newConnectionPage()
 {
@@ -1089,8 +1174,7 @@ void MainWindow::newConnectionPage()
         const QString toolTip(senderObject->property("toolTipString").toString());
         if (!toolTip.isEmpty()) {
             QToolTip::showText(m_addressNavigator->pos() + pos() + QPoint(m_addressNavigator->width() / 2,
-                                                                          m_addressNavigator->height() / 2),
-                            toolTip, this);
+                               m_addressNavigator->height() / 2), toolTip, this);
         }
     }
 #endif
@@ -1112,13 +1196,6 @@ int MainWindow::currentRemoteView() const
     return m_currentRemoteView;
 }
 
-void MainWindow::expandTreeViewItems()
-{
-    metaObject()->invokeMethod(m_remoteDesktopsTreeView, "expandAll", Qt::QueuedConnection);
-    if (m_remoteDesktopsNewConnectionTabTreeView)
-        metaObject()->invokeMethod(m_remoteDesktopsNewConnectionTabTreeView, "expandAll", Qt::QueuedConnection);
-}
-
 void MainWindow::updateFilter(const QString &text)
 {
     m_remoteDesktopsModelProxy->setFilterRegExp(QRegExp("IGNORE|" + text, Qt::CaseInsensitive, QRegExp::RegExp));
@@ -1135,29 +1212,36 @@ void MainWindow::createDockWidget()
     remoteDesktopsDockWidget->setMinimumWidth(fontMetrics.width("vnc://192.168.100.100:6000"));
     actionCollection()->addAction("remote_desktop_dockwidget",
                                   remoteDesktopsDockWidget->toggleViewAction());
-    m_remoteDesktopsTreeView = new QTreeView(remoteDesktopsDockLayoutWidget);
-    RemoteDesktopsModel *remoteDesktopsModel = new RemoteDesktopsModel(this);
 
+    m_dockWidgetTableView = new QTableView(remoteDesktopsDockLayoutWidget);
+    RemoteDesktopsModel *remoteDesktopsModel = new RemoteDesktopsModel(this);
     m_remoteDesktopsModelProxy = new QSortFilterProxyModel(this);
     m_remoteDesktopsModelProxy->setSourceModel(remoteDesktopsModel);
-    m_remoteDesktopsModelProxy->setFilterKeyColumn(0);
     m_remoteDesktopsModelProxy->setFilterRole(10002);
+    m_dockWidgetTableView->setModel(m_remoteDesktopsModelProxy);
 
-    connect(remoteDesktopsModel, SIGNAL(modelReset()), this, SLOT(expandTreeViewItems()));
-    m_remoteDesktopsTreeView->setModel(m_remoteDesktopsModelProxy);
-    m_remoteDesktopsTreeView->header()->hide();
-    m_remoteDesktopsTreeView->expandAll();
-    m_remoteDesktopsTreeView->setItemsExpandable(true);
-    m_remoteDesktopsTreeView->setRootIsDecorated(false);
-    connect(m_remoteDesktopsTreeView, SIGNAL(doubleClicked(const QModelIndex &)),
-                                    SLOT(openFromDockWidget(const QModelIndex &)));
+    m_dockWidgetTableView->setShowGrid(false);
+    m_dockWidgetTableView->verticalHeader()->hide();
+    m_dockWidgetTableView->verticalHeader()->setDefaultSectionSize(
+        m_dockWidgetTableView->fontMetrics().height() + 2);
+    m_dockWidgetTableView->horizontalHeader()->hide();
+    m_dockWidgetTableView->horizontalHeader()->setStretchLastSection(true);
+    // hide all columns, then show the one we want
+    for (int i=0; i<m_remoteDesktopsModelProxy->columnCount(); i++) {
+        m_dockWidgetTableView->hideColumn(i);
+    }
+    m_dockWidgetTableView->showColumn(RemoteDesktopsModel::Title);
+    m_dockWidgetTableView->sortByColumn(RemoteDesktopsModel::Title, Qt::AscendingOrder);
+
+    connect(m_dockWidgetTableView, SIGNAL(doubleClicked(const QModelIndex &)),
+            SLOT(openFromRemoteDesktopsModel(const QModelIndex &)));
 
     KLineEdit *filterLineEdit = new KLineEdit(remoteDesktopsDockLayoutWidget);
     filterLineEdit->setClickMessage(i18n("Filter"));
     filterLineEdit->setClearButtonShown(true);
     connect(filterLineEdit, SIGNAL(textChanged(const QString &)), SLOT(updateFilter(const QString &)));
     remoteDesktopsDockLayout->addWidget(filterLineEdit);
-    remoteDesktopsDockLayout->addWidget(m_remoteDesktopsTreeView);
+    remoteDesktopsDockLayout->addWidget(m_dockWidgetTableView);
     remoteDesktopsDockWidget->setWidget(remoteDesktopsDockLayoutWidget);
     addDockWidget(Qt::LeftDockWidgetArea, remoteDesktopsDockWidget);
 }
