@@ -49,7 +49,7 @@ class TabbedViewWidget;
 class QScrollArea;
 class QModelIndex;
 class QSortFilterProxyModel;
-class QTreeView;
+class QTableView;
 
 #ifdef TELEPATHY_SUPPORT
 class TubesManager;
@@ -67,10 +67,11 @@ public:
     int currentRemoteView() const;
 
 public slots:
-    void newConnection(const KUrl &newUrl = KUrl(), bool switchFullscreenWhenConnected = false);
+    void newConnection(const KUrl &newUrl = KUrl(), bool switchFullscreenWhenConnected = false, const QString &tabName = QString());
 
 protected:
     virtual void closeEvent(QCloseEvent *event);
+    bool eventFilter(QObject *obj, QEvent *event); // checks for close events on fs window
     virtual void saveProperties(KConfigGroup &group);
     void saveHostPrefs(RemoteView *view = 0);
 
@@ -100,16 +101,17 @@ private slots:
     void tabChanged(int index);
     QWidget* newConnectionWidget();
     void newConnectionPage();
-    void openFromDockWidget(const QModelIndex &index);
-    void expandTreeViewItems();
+    void openFromRemoteDesktopsModel(const QModelIndex &index);
     void updateFilter(const QString &text);
     void createDockWidget();
+    void showConnectionContextMenu(const QPoint &pos);
+    void saveConnectionListSort(const int logicalindex, const Qt::SortOrder order);
 
 private:
     void setupActions();
     void loadAllPlugins();
     RemoteViewFactory *createPluginFromService(const KService::Ptr &service);
-
+    void showSettingsDialog(const QString &url);
     QScrollArea *createScrollArea(QWidget *parent, RemoteView *remoteView);
 
     QWidget *m_fullscreenWindow;
@@ -126,15 +128,12 @@ private:
     QList<RemoteView *> m_remoteViewList;
     QMap<int, RemoteViewFactory *> m_remoteViewFactories;
 
-    int m_topBottomBorder; // tabwidget borders
-    int m_leftRightBorder;
-
     int m_currentRemoteView;
     bool m_switchFullscreenWhenConnected;
 
     SystemTrayIcon *m_systemTrayIcon;
-    QTreeView *m_remoteDesktopsTreeView;
-    QTreeView *m_remoteDesktopsNewConnectionTabTreeView;
+    QTableView *m_dockWidgetTableView;
+    QTableView *m_newConnectionTableView;
     QSortFilterProxyModel *m_remoteDesktopsModelProxy;
 #ifdef TELEPATHY_SUPPORT
     Tp::SharedPtr<TubesManager> m_tubesManager;
